package com.informationkinematics.imsforward;

import java.io.*;
import java.util.*;

/**
 * Title:        CTLIOController<br>
 * Description:  This helper handles the reading and writing of the IMS mail server's control file.<br>
 * Copyright:    Copyright (c) 2001 by Information Kinematics under GNU General Public License
 *               (see package description for more information)<br>
 * Company:      Information Kinematics<br>
 * @author Florian Steinsiepe, Adrian Zaugg
 * @version 0.9.2
 */

public class CTLIOController
{
	/**
	 * Constructor is inactive.
	 */
	public CTLIOController() {}

	/**
	 * Reads the mail server's controller file or its inactivated version.
	 *
	 * @param file java.io.File
	 * @param sEmail java.lang.String[1] - provides the pointer, not only the value
	 *                                     of the client's email address string
	 * @return boolean - whether the mails shall be removed from the server
	 *                   after being forwarded
	 */
	public boolean readCTLFile(File file, String[] sEmail) throws IOException
	{
		BufferedReader br = null;
		boolean bDiscard = false;
		br = new BufferedReader(new FileReader(file));
		String sNextLine = "";
		while ((sNextLine = br.readLine()) != null)
		{
			sNextLine = sNextLine.trim();
			if (sNextLine.toLowerCase().startsWith("forwardto"))
				sEmail[0] = readStringParameter(sNextLine);
			else if (sNextLine.toLowerCase().startsWith("discard"))
				bDiscard = readBooleanParameter(sNextLine);
		}
		br.close();
		return bDiscard;
	}

	/**
	 * Writes the contents of the mail server's controller file or its inactivated version.
	 *
	 * @param file java.io.File
	 * @param sEmail java.lang.String[1] - provides the pointer, not only the value
	 *                                     of the client's email address string
	 * @param bDiscard boolean - whether the mails shall be removed from the server
	 *                           after being forwarded
	 */
	public void writeCTLFile(File file, String sEmail, boolean bDiscard) throws IOException
	{
		if (file.exists())
			deleteFile(file);
		BufferedWriter bw = new BufferedWriter(new FileWriter(file));
		bw.newLine();
		bw.write("[AutoForward]");
		bw.newLine();
		bw.write("\tForwardTo = " + sEmail);
		bw.newLine();
		bw.newLine();
		bw.write("[Delivery]");
		bw.newLine();
		bw.write("\tDiscard = " + ((bDiscard) ? "Yes" : "No"));
		bw.newLine();
		bw.close();
	}

	/**
	 * Returns the parameter value of the given "key=value" string as a String.
	 *
	 * @param sLine java.lang.String
	 * @return java.lang.String - the value, i.e. the right part of the input
	 */
	private String readStringParameter(String sLine)
	{
		String sValue = "";
		StringTokenizer st = new StringTokenizer(sLine, "=");
		try {
			String sDummy = st.nextToken();
			sValue = st.nextToken().trim();
		} catch (NoSuchElementException e) {}
		return sValue;
	}

	/**
	 * Returns the parameter value of the given "key=value" string as a boolean.
	 *
	 * @param sLine java.lang.String
	 * @return boolean - the value, i.e. the right part of the input
	 */
	private boolean readBooleanParameter(String sLine)
	{
		boolean bValue = false;
		StringTokenizer st = new StringTokenizer(sLine, "=");
		try {
			String sDummy = st.nextToken();
			sDummy = st.nextToken().toLowerCase().trim();
			if (sDummy.equals("yes"))
				bValue = true;
		} catch (NoSuchElementException e) {}
		return bValue;
	}

	/**
	 * Deletes the given file. If the operation fails, the action is retried 10 times,
	 * every 200 milliseconds. In case this isn't possible, a manual IOException is thrown.
	 *
	 * @param file java.io.File
	 */
	public void deleteFile(File file) throws IOException
	{
		int ctr = 0;
		while (!file.delete() && (ctr++ < 10))
		{
			try {
				Thread.sleep(200L);
			} catch (InterruptedException e) {}
		}
		//if the deleting failed 10 times, notify the caller
		if (ctr >= 10)
			throw new IOException();
	}
}