package com.informationkinematics.imsforward;

import java.util.*;
import java.io.*;

/**
 * Title:        Environment (based on code by OwenR's Environment.java)<br>
 * Description:  Finds the value of a given OS environment variable. Note: This class is
 *               platform-dependent and runs on Windows or Unix types only!<br>
 * Copyright:    Copyright (c) 2001 by Information Kinematics under GNU General Public License
 *               (see package description for more information)<br>
 * Company:      Information Kinematics<br>
 * @author Florian Steinsiepe, Adrian Zaugg
 * @version 0.9.2
 */

public class Environment
{
	/**
	 * Constructor is inactive.
	 */
	public Environment() {}

	/**
	 * Returns the value of the OS' environment variable named sEnvVarName.
	 * Supported OS' are: Unix and derivatives, Windows family. In case the OS
	 * isn't a Windows family member, doesn't understand the "env" command and
	 * doesn't store environment variables in the format "$VARNAME", an
	 * IOException is thrown.
	 *
	 * @param sEnvVarName java.lang.String
	 */
	public String getEnvVariable(String sEnvVarName) throws EnvVarNotFoundException, IOException
	{
		String sNextLine = "";
		String sEnvValue = "";
		int i = 0;
		Process theProcess = null;
		BufferedReader stdInput = null;

		//Figure out what kind of system we are running on
		// 0 = windows
		// 1 = non-windows (considered to be unix)
		boolean bIsWindows = isWindows();

		if (bIsWindows)
			theProcess = Runtime.getRuntime().exec("cmd /C set");
		else
			theProcess = Runtime.getRuntime().exec("env");

		stdInput = new BufferedReader(new InputStreamReader(theProcess.getInputStream()));
		boolean bValueFound = false;

		while ((sNextLine = stdInput.readLine()) != null)
		{
			if (sNextLine != null)
			{
				if (sNextLine.startsWith(sEnvVarName))
				{
					//parse the value
					StringTokenizer token = new StringTokenizer(sNextLine, "=");

					sEnvValue = token.nextToken();
					//make sure that the name value pair does not just
					//have the value in 'name' as a substring
					if (sEnvValue.equals(sEnvVarName))
					{
						sEnvValue = token.nextToken();
						sEnvValue.trim();
						bValueFound = true;
						break;
					}
					else
						sEnvValue = "";
				}
			}
		}

		stdInput.close();
		theProcess.destroy();
		if ((sEnvValue.length()==0) && (!bValueFound))
		{
			//in case the environment variable is empty or the variable key contains
			//the value, build an OS-specific exception message and throw it
			String sIdent = "";
			sEnvValue = "";
			if (bIsWindows)
				sIdent = "%";
			else
				sIdent = "$";

			String sFormattedEnvVarName = sIdent + sEnvVarName + ((bIsWindows)? "%": "");
			throw new EnvVarNotFoundException ("No such environement variable " + sFormattedEnvVarName +
				    " defined.", sFormattedEnvVarName);
		}

		return sEnvValue;
	}

	/**
	 * Returns the type of the operation system, as follows:<br>
	 * true: Windows family
	 * false: Others<br>
	 *
	 * @return boolean - true if OS name contains "Win", else false
	 */
	private boolean isWindows()
	{
		Properties p = System.getProperties();
		String sOSName = p.getProperty("os.name");
		int i = 0;

		if ((i = sOSName.indexOf("Win")) != -1)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
 }