package com.informationkinematics.imsforward;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.mail.*;

/**
 * Title:        Setup<br>
 * Description:  Performs the user authentication, reads preexisting ims.ctl file contents, if any,
 *               and shows the second screen of the GUI.<br>
 * Copyright:    Copyright (c) 2001 by Information Kinematics under GNU General Public License
 *               (see package description for more information)<br>
 * Company:      Information Kinematics<br>
 * @author Florian Steinsiepe, Adrian Zaugg
 * @version 0.9.2
 */


public class Setup extends Littles
{
	/**
	 * doGet calls doPost using its parameters.
	 *
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 */
    public void doGet(HttpServletRequest req, HttpServletResponse res)  throws ServletException, IOException
    {
        // We don't get, we post.
        doPost(req, res);
    }

	/**
	 * doPost reads the client's input and tries to authenticate the user on the POP3 host.
	 * An eventually existing ims.ctl file's contents are read.<br>
	 *
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 */
    public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
    {
        String sIMSPath = "";
        String sError = "";
        boolean bForwardingIsActive = false, bDiscard = false;
        String[] sEmail = {""};
        int iNewMailCount = 0;

        // Get username parameter
        String sUserName = req.getParameter("username");
        String sPassWord = req.getParameter("password");

        // Check login and number of new mails
        try {
			iNewMailCount = new Authentication (sUserName, sPassWord, sPOP3Host).getNewMailCount();

			// complete IMSPath
			sIMSPath = this.getIMSPath(sUserName);

			// read user's IMS.CTL resp. IMS.CTL.inactive File
			File fIMSFileA = new File(sIMSPath + IMSFILE_ACTIVE);
			File fIMSFileI = new File(sIMSPath + IMSFILE_INACTIVE);
			CTLIOController ioC = new CTLIOController();

			if (fIMSFileA.exists())
			{
				bForwardingIsActive = true;
				bDiscard = ioC.readCTLFile(fIMSFileA, sEmail);
			}
			else if (fIMSFileI.exists())
			{
				bForwardingIsActive = false;
				bDiscard = ioC.readCTLFile(fIMSFileI, sEmail);
			}
        } catch (AuthenticationFailedException ae) {
			// authentication failed!
			sError = "\"" + ae.getMessage() + "\"";
			sError += "<br><br>Username and/or password incorrect, or mailbox locked.<br>Please try again!";
        } catch (FolderClosedException fe) {
			// mailbox locked
			sError = "\"" + fe.getMessage() + "\"";
			sError += "<br><br>Mailbox locked.<br>Close any mailreader accessing your mailbox or try again later!";
        } catch (MessagingException me) {
			// other error
			sError = "\"" + me.getMessage() + "\"";
			Exception internalEx = me.getNextException();
			if (internalEx instanceof java.net.ConnectException)
			{
				if ((sPOP3Host == null) || (sPOP3Host.equals("")))
					sError += "<br><br>No POP3 server has been defined in imsforward.properties yet." +
						    " Please read the ReadMe.txt file how to setup IMSForward correctly.";
				else
					sError += "<br><br>No connection to POP3 host '" + sPOP3Host + "'. Either the server is down "+
							" or the address in imsforward.properties is wrong.";
			}
			else if (internalEx instanceof java.net.UnknownHostException)
				sError += "<br><br>Host '" + sPOP3Host + "' is unknown. Make sure the host name is correct.";
			else if (internalEx instanceof java.net.NoRouteToHostException)
				sError += "<br><br>An error occurred while attempting to connect a socket to the remote address '" +
					    sPOP3Host + "' and port 110. Typically, the remote host cannot be reached because of an " +
						"intervening firewall, or if an intermediate router is down or misconfigured.";
			else
				sError += "<br><br>Maybe a .jar of the distribution is not available or some ambiguous error occured.<br>" +
					"Please try again or ask for assistance!";
        } catch (EnvVarNotFoundException e) {
			// environment variable not found
			sError = "\"" + e.getMessage() + "\"<br>Check your environment variable set in the startup file 'imsforward.properties'.";
        } catch (IOException ioe) {
			// either some error in Environment-Class or the path in the properties file is wrong
			sError = "\"" + ioe.getMessage() + "\"";
			sError += "<br><br>The value of 'IMSPath' in the imsforward.properties file is may be wrong." +
					"<br>Check the properties file!";
        }

        // Present Setup Window and fill in actual values from .CTL-File.
        res.setContentType("text/html");
        PrintWriter out = res.getWriter();
        if (sError.length() == 0 )
		{
			// no error in authentication
			out.println(HTMLCode.SETUP_1 + ((bForwardingIsActive)?"checked":"") +
					  HTMLCode.SETUP_2 + sEmail[0] +
					  HTMLCode.SETUP_3 + ((bDiscard)?"checked":""));

			// write number of new mails if any and pass sUserName, iNewMailCount and sIMSPath.
			out.println(HTMLCode.SETUP_4 + sUserName +
					  HTMLCode.SETUP_5 + iNewMailCount +
					  HTMLCode.SETUP_6 + sIMSPath +
					  HTMLCode.SETUP_7);

			// builds the grammatically correct "number of mails in box" output
			if (iNewMailCount == 1)
				out.println("<i>There is one new message on the server!</i>");
			else if (iNewMailCount > 1)
				out.println("<i>There are " + iNewMailCount + " new messages on the server!</i>");
			else
				out.println("<i>No new messages on the server!</i>");

			// finishes HTML output
			out.println(HTMLCode.SETUP_8);
        }
        else
		{
			//  some error occured
			out.println(HTMLCode.ERROR_1 + sError +
					  HTMLCode.ERROR_2);
        }
        out.close();
    }
}